<?php
/**
rb-forms
Version: 1.0
By Upzilla.co Team

Contact: https://www.upzilla.co/rb-forms/
Created: 2022
Licensed under the GPLv2 license - http://opensource.org/licenses/gpl-2.0.php

Copyright (c) 2022, Upzilla.co All rights reserved.
Available only in https://www.upzilla.co
https://www.upzilla.co
 **/

namespace upz\rbforms;

if (!defined('WPINC')) {
    exit;
}

class API
{

    public function __construct()
    {
        $this->init();
        $this->hooks();
    }

    private function getPrefix()
    {
        return Config::PREFIX . '/' . Config::PREFIX_VERSION;
    }

    public function init()
    {

    }

    public function hooks()
    {

        add_action('rest_api_init', function () {

            register_rest_route($this->getPrefix(), '/form/(?P<form_id>\d+)', array(
                array(
                    'methods'             => 'POST',
                    'callback'            => array($this, 'formSubmitHandler'),
                    'permission_callback' => array($this, 'isCorrectFormIdInRequest'),
                ),
                array(
                    'methods'             => 'GET',
                    'callback'            => array($this, 'formRecords'),
                    'permission_callback' => array($this, 'isCorrectFormIdInRequestAndUserCanEditForm'),
                ),

                array(
                    'methods'             => 'DELETE',
                    'callback'            => array($this, 'deleteAllRecords'),
                    'permission_callback' => array($this, 'isCorrectFormIdInRequestAndUserCanEditForm'),
                ),
            ));

            register_rest_route($this->getPrefix(), '/form/(?P<form_id>\d+)/unread', array(
                array(
                    'methods'             => 'GET',
                    'callback'            => array($this, 'formUnreadRecords'),
                    'permission_callback' => array($this, 'isCorrectFormIdInRequestAndUserCanEditForm'),
                ),
            ));

            register_rest_route($this->getPrefix(), '/record/(?P<record_id>\d+)/', array(
                array(
                    'methods'             => 'GET',
                    'callback'            => array($this, 'getRecord'),
                    'permission_callback' => array($this, 'isCorrectRecordIdInRequestAndUser'),
                ),
                array(
                    'methods'             => 'DELETE',
                    'callback'            => array($this, 'deleteRecord'),
                    'permission_callback' => array($this, 'isCorrectRecordIdInRequestAndUserCanEditForm'),
                ),
            ));

            /* delete many records   */
            register_rest_route($this->getPrefix(), '/record/(?P<record_ids>[\d, ]+)/', array(
                array(
                    'methods'             => 'DELETE',
                    'callback'            => array($this, 'deleteRecords'),
                    'permission_callback' => array($this, 'isCorrectRecordsIdInRequestAndUserCanEditForm'),
                ),
            ));

            register_rest_route($this->getPrefix(), '/options/(?P<form_id>\d+)/', array(
                array(
                    'methods'             => 'GET',
                    'callback'            => array($this, 'getOptions'),
                    'permission_callback' => array($this, 'isCorrectFormIdInRequestAndUserCanEditForm'),
                ),
                array(
                    'methods'             => 'POST',
                    'callback'            => array($this, 'setOptions'),
                    'permission_callback' => array($this, 'isCorrectFormIdInRequestAndUserCanEditForm'),
                ),
            ));

        });
    }

    /**
     * isUserLogin check if user login in
     *
     * @param [WP_REST_Response] $request
     * @return boolean
     */
    public function isUserLogin($request)
    {
        return is_user_logged_in() && get_current_user_id();
    }

    /**
     * Check if user can edit form and records
     *
     * @param [WP_REST_Response] $request
     * @return boolean
     */
    public function isUserCanEditPost($request)
    {
        return $this->isUserLogin($request) && current_user_can('edit_posts');
    }

    /**
     * Check form_id in request is correct and user can edit form
     *
     * @param [WP_REST_Response] $request
     * @return boolean
     */
    public function isCorrectFormIdInRequestAndUserCanEditForm($request)
    {
        return $this->isUserCanEditPost($request) && $this->isCorrectFormIdInRequest($request);
    }

    /**
     * Check form_id in request is correct
     *
     * @param [WP_REST_Response] $request
     * @return boolean
     */
    public function isCorrectFormIdInRequest($request)
    {

        $form_id = (int) $request->get_param('form_id');

        if (!$form_id) {
            return false;
        }

        return $this->isCorrectFormId($form_id);
    }

    /**
     * Check param form_id  is correct
     *
     * @param  $form_id
     * @return boolean
     */

    public function isCorrectFormId($form_id)
    {
        $form_id = (int) $form_id;

        if (!$form_id) {
            return false;
        }

        $post_status = get_post_status($form_id);
        if ('publish' == $post_status || 'auto-draft' == $post_status || 'draft' == $post_status) {
            return true;
        }

        return false;
    }

    public function isCorrectRecordIdInRequestAndUserCanEditForm($request)
    {
        return $this->isUserCanEditPost($request) && $this->isCorrectRecordIdInRequest($request);
    }

    public function isCorrectRecordIdInRequestAndUser($request)
    {
        return $this->isUserLogin($request) && $this->isCorrectRecordIdInRequest($request);
    }

    public function isCorrectRecordsIdInRequestAndUserCanEditForm($request)
    {
        return $this->isUserCanEditPost($request) && $this->isCorrectRecordsIdInRequest($request);
    }

    public function isCorrectRecordIdInRequest($request)
    {
        if (!isset($request['record_id'])) {
            return false;
        }

        $record_id = (int) $request['record_id'];

        if (!$record_id) {
            return false;
        }

        return $this->isCorrectRecordId($record_id);
    }

/**
 * checks the accuracy of the provided record IDs.
 * If the IDs are correct, it returns true, otherwise, it returns false.
 *
 * @param request
 *
 * @return bool
 */

    public function isCorrectRecordsIdInRequest($request)
    {
        if (!isset($request['record_ids'])) {
            return false;
        }

        $record_ids = $request['record_ids'];

        if (!$record_ids) {
            return false;
        }

        $a_ids = \explode(',', $record_ids);
        if (!is_array($a_ids) || !count($a_ids)) {
            return false;
        }

        foreach ($a_ids as $key => $record_id) {
            if (!$this->isCorrectRecordId($record_id)) {
                return false;
            }

        }

        return true;
    }

    public function isCorrectRecordId($record_id)
    {
        $record_id = (int) $record_id;

        if (!$record_id) {
            return false;
        }

        if ('publish' == get_post_status($record_id)) {
            return true;
        }

        return false;
    }

    /*
    ##################
    API commands
    ##################
     */
    public function getRecord($request)
    {

        $record_id = (int) $request->get_param('record_id');

        if (!$this->isCorrectRecordId($record_id)) {
            return new \WP_Error('403', 'EMPTY_RECORD_ID', "Record_id is empty");
        }

        $record = get_post($record_id);

        if (!is_a($record, 'WP_Post')) {
            return new \WP_Error('403', 'RECORD_INCORRECT', "Record is incorrect");
        }

        if ($record->post_type != RB_FORMS_TYPE_POST_ROWS) {
            return new \WP_Error('403', 'RECORD_TYPE_INCORRECT', "Record type is incorrect");
        }

        $sent_status = get_post_meta($record->ID, '_rbforms_sent', true);
        if ($sent_status === '') {
            $sent_status = 3;
        } else {
            $sent_status = (int) $sent_status;
        }

        $sent_status_text = isset(Config::SENT_STATUS[$sent_status]) ? Config::SENT_STATUS[$sent_status] : "Unknown status";

        $data = array(
            'id'    => $record->ID,
            'title' => $record->post_title,
            'date'  => $record->post_date,
            'sent'  => $sent_status_text,
            'userip'  => get_post_meta($record->ID, '_rbforms_userip', true),
            'read'  => get_post_meta($record->ID, '_rbforms_read', true) ? true : false,
            'rows'  => get_post_meta($record->ID, '_rbforms_rows', true),
        );

        if (!$data['read']) {
            update_post_meta($record->ID, '_rbforms_read', true, false);
        }

        return new \WP_REST_Response($data, 200);
    }

    public function deleteAllRecords($request)
    {

        $form_id = (int) $request->get_param('form_id');

        if (!$this->isCorrectFormId($form_id)) {
            return new \WP_Error('403', 'EMPTY_FORMID', "Form_id is empty");
        }

        $form = get_post($form_id);

        if (!is_a($form, 'WP_Post')) {
            return new \WP_Error('403', 'FORM_INCORRECT', "Form is incorrect");
        }

        if ($form->post_type != RB_FORMS_TYPE_POST_FORMS) {
            return new \WP_Error('403', 'FORM_TYPE_INCORRECT', "Form type is incorrect");
        }

        $args = array(
            'nopaging'    => true,
            //     'posts_per_page' => 100,

            'order'       => 'DESC',
            'orderby'     => 'date',
            'post_parent' => $form_id,
            'post_status' => null,
            'post_type'   => RB_FORMS_TYPE_POST_ROWS,
            //  'meta_key'  => '_rbforms_rows',
        );

        $records = get_children($args);

        $data = array();

        if (!count($records)) {
            return new \WP_REST_Response(array(), 200);
        }

        foreach ($records as $rec) {
            $data[] = $rec->ID;
            wp_delete_post($rec->ID, true);
        }
        return new \WP_REST_Response($data, 200);
    }

    public function deleteRecords($request)
    {
        $record_ids = $request->get_param('record_ids');

        if (!$record_ids) {
            return new \WP_Error('403', 'RECORD_IDS_EMPTY', "Record_ids is empty");
        }

        $a_ids = \explode(',', $record_ids);
        if (!is_array($a_ids) || !count($a_ids)) {
            return new \WP_Error('403', 'RECORD_IDS_INCORRECT', "Record_ids is incorrect");
        }

        $a_ids = array_unique($a_ids);

        $delete_ids = [];

        foreach ($a_ids as $key => $record_id) {
            $record_id = (int) $record_id;

            if (!$this->isCorrectRecordId($record_id)) {
                return new \WP_Error('403', 'RECORD_IDS_INCORRECT', "Record_ids is incorrect");
            }
            $record = get_post($record_id);
            if (!is_a($record, 'WP_Post')) {
                return new \WP_Error('403', 'RECORD_INCORRECT', "Record is incorrect");
            }

            if ($record->post_type != RB_FORMS_TYPE_POST_ROWS) {
                return new \WP_Error('403', 'RECORD_TYPE_INCORRECT', "Record type is incorrect");
            }

            $deleteResult = wp_delete_post($record->ID);

            if (!is_a($deleteResult, 'WP_Post')) {
                return new \WP_Error('403', 'RECORD_DELETE_INCORRECT', "Record delete is unsuccessful");
            }
            $delete_ids[] = $deleteResult->ID;
        }

        $result = self::success_response($delete_ids);
        // $result['data'] = $delete_ids;

        return new \WP_REST_Response($result, 200);
    }

    public function deleteRecord($request)
    {
        $record_id = (int) $request->get_param('record_id');

        if (!$this->isCorrectRecordId($request)) {
            return new \WP_Error('403', 'RECORD_ID_EMPTY', "Record_id is empty");
        }

        $record = get_post($record_id);

        if (!is_a($record, 'WP_Post')) {
            return new \WP_Error('403', 'RECORD_INCORRECT', "Record is incorrect");
        }

        if ($record->post_type != RB_FORMS_TYPE_POST_ROWS) {
            return new \WP_Error('403', 'RECORD_TYPE_INCORRECT', "Record type is incorrect");
        }

        $deleteResult = wp_delete_post($record->ID);

        if (!is_a($deleteResult, 'WP_Post')) {
            return new \WP_Error('403', 'RECORD_DELETE_INCORRECT', "Record delete is unsuccessful");
        }

        $data = array(
            'id' => $deleteResult->ID,
        );

        return new \WP_REST_Response($data, 200);
    }

    public function formUnreadRecords($request)
    {
        return $this->formRecords($request, true);
    }

    public function formRecords($request, $unreadOnly = false)
    {

        $form_id = (int) $request->get_param('form_id');

        if (!$this->isCorrectFormId($form_id)) {
            return new \WP_Error('403', 'EMPTY_FORMID', "Form_id is empty");
        }

        $form = get_post($form_id);

        if (!is_a($form, 'WP_Post')) {
            return new \WP_Error('403', 'FORM_INCORRECT', "Form is incorrect");
        }

        if ($form->post_type != RB_FORMS_TYPE_POST_FORMS) {
            return new \WP_Error('403', 'FORM_TYPE_INCORRECT', "Form type is incorrect");
        }

        $args = array(
            //'posts_per_page' => 100,
            'nopaging'    => true,

            'order'       => 'DESC',
            'orderby'     => 'date',
            'post_parent' => $form_id,
            'post_status' => null,
            'post_type'   => RB_FORMS_TYPE_POST_ROWS,
        );

        if ($unreadOnly) {
            $args['meta_key']   = '_rbforms_read';
            $args['meta_value'] = 0;
        }

        $records = get_children($args);

        $data = array();

        if (!count($records)) {
            return new \WP_REST_Response(array(), 200);
        }

        foreach ($records as $rec) {

            $data[] = array(
                'id'    => $rec->ID,
                'title' => $rec->post_title,
                'date'  => $rec->post_date,
                'read'  => get_post_meta($rec->ID, '_rbforms_read', true) ? true : false,
            );

        }
        return new \WP_REST_Response($data, 200);
    }

    public static function get_the_user_ip()
    {
        if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
            //check ip from share internet
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            //to check ip is pass from proxy
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }
        if($ip=='::1') $ip="127.0.0.1";
        return  $ip;
    }

    public function formSubmitHandler($request)
    {

        if (!is_a($request, 'WP_REST_Request')) {
            return new \WP_Error('403', 'REQUEST_INCORRECT', "Request is incorrect");
        }

        $params = $request->get_url_params();

        $form_id = (int) $request->get_param('form_id');

        $form_options = get_post_meta($form_id, Config::KEY_OPTIONS, true);

        $sent_status = 3;
        if (isset($form_options['notification']) && $form_options['notification']) {
            $sent_status = 0;
        }

        $jsonData = $request->get_json_params();

        $dataToSend = array();

        if (!empty($jsonData['name'])) {
            $dataToSend['name'] = sanitize_text_field($jsonData['name']);
        }

        if (!empty($jsonData['email'])) {
            $dataToSend['email'] = sanitize_email($jsonData['email']);
        }

        if (!empty($jsonData['subject'])) {
            $dataToSend['subject'] = sanitize_text_field($jsonData['subject']);
        }

        if (!empty($jsonData['message'])) {
            $dataToSend['message'] = sanitize_textarea_field($jsonData['message']);
        }

        if (!empty($jsonData['priority'])) {
            $dataToSend['priority'] = sanitize_text_field($jsonData['priority']);
        }

        if (!empty($jsonData['phone'])) {
            $dataToSend['phone'] = sanitize_text_field($jsonData['phone']);
        }

        if (!empty($jsonData['agreement'])) {
            $dataToSend['agreement'] = isset($jsonData['agreement']) && $jsonData['agreement'];
        }

        $new_post_id = wp_insert_post(array(
            'post_type'      => RB_FORMS_TYPE_POST_ROWS,
            'post_title'     => 'RB Form record from ' . wp_date(__('Y/m/d g:i:s a', 'forms-rb')),
            'post_content'   => 'oo',
            'post_status'    => 'publish',
            'comment_status' => 'closed', // if you prefer
            'ping_status' => 'closed',
            'post_parent'    => $form_id,
            'meta_input'     => array(
                '_rbforms_rows'    => $dataToSend,
                '_rbforms_formid'  => $form_id,
                '_rbforms_read'    => 0,
                '_rbforms_sent'    => $sent_status,
                '_rbforms_userip' => self::get_the_user_ip(),
            ),
        ));

        $dataToSend['post_id'] = $new_post_id;

        $result = self::success_response(__("Form submitted successfully", 'forms-rb'));

        return new \WP_REST_Response($result, 200);
    }

    public function getOptions($request)
    {

        $form_id = (int) $request->get_param('form_id');

        if (!$this->isCorrectFormId($form_id)) {
            return new \WP_Error('403', 'EMPTY_FORMID', "Form_id is empty");
        }

        $form = get_post($form_id);

        if (!is_a($form, 'WP_Post')) {
            return new \WP_Error('403', 'FORM_INCORRECT', "Form is incorrect");
        }

        if ($form->post_type != RB_FORMS_TYPE_POST_FORMS) {
            return new \WP_Error('403', 'FORM_TYPE_INCORRECT', "Form type is incorrect");
        }

        $options_raw = get_post_meta($form->ID, Config::KEY_OPTIONS, true);

        $options = wp_parse_args($options_raw, Config::DEFAULT_FORM_OPTIONS);

        if (!$options['mailTo']) {
            $options['mailTo'] = get_option('admin_email');
        }

        $result = self::success_response();

        $result['options'] = $options;

        return new \WP_REST_Response($result, 200);
    }

    public function setOptions($request)
    {

        $form_id = (int) $request->get_param('form_id');

        if (!$this->isCorrectFormId($form_id)) {
            return new \WP_Error('403', 'EMPTY_FORMID', "Form_id is empty");
        }

        $form = get_post($form_id);

        if (!is_a($form, 'WP_Post')) {
            return new \WP_Error('403', 'FORM_INCORRECT', "Form is incorrect");
        }

        if ($form->post_type != RB_FORMS_TYPE_POST_FORMS) {
            return new \WP_Error('403', 'FORM_TYPE_INCORRECT', "Form type is incorrect");
        }

        $newOptions   = array();
        $inputOptions = $request->get_json_params();

        /* sanitize */
        $fields = null;
        if (isset($inputOptions['fields']) && count($inputOptions['fields'])) {
            $fields = array_filter(array_map('sanitize_key', $inputOptions['fields']));
            foreach ($fields as $name => $value) {
                $fields[$name] = sanitize_text_field($value);
            }
            $newOptions['fields'] = $fields;
        }

        if (isset($inputOptions['useDialog'])) {
            $newOptions['useDialog'] = $inputOptions['useDialog'] === true;
        }

        if (isset($inputOptions['buttonWithText'])) {
            $newOptions['buttonWithText'] = $inputOptions['buttonWithText'] === true;
        }

        if (isset($inputOptions['buttonLabel'])) {
            $newOptions['buttonLabel'] = sanitize_text_field($inputOptions['buttonLabel']);
        }

        if (isset($inputOptions['buttonIcon'])) {
            $newOptions['buttonIcon'] = sanitize_text_field($inputOptions['buttonIcon']);
        }

        if (isset($inputOptions['buttonColor'])) {
            $newOptions['buttonColor'] = sanitize_text_field($inputOptions['buttonColor']);
        }

        if (isset($inputOptions['buttonSize'])) {
            $newOptions['buttonSize'] = sanitize_text_field($inputOptions['buttonSize']);
        }

        if (isset($inputOptions['dialogTitle'])) {
            $newOptions['dialogTitle'] = sanitize_text_field($inputOptions['dialogTitle']);
        }

        if (isset($inputOptions['dialogDesc'])) {
            $newOptions['dialogDesc'] = sanitize_text_field($inputOptions['dialogDesc']);
        }

        if (isset($inputOptions['dialogButtonClose'])) {
            $newOptions['dialogButtonClose'] = sanitize_text_field($inputOptions['dialogButtonClose']);
        }

        if (isset($inputOptions['dialogButtonSubmit'])) {
            $newOptions['dialogButtonSubmit'] = sanitize_text_field($inputOptions['dialogButtonSubmit']);
        }

        if (isset($inputOptions['notification'])) {
            $newOptions['notification'] = $inputOptions['notification'] == true;
        }

        if (isset($inputOptions['mailSubject'])) {
            $newOptions['mailSubject'] = sanitize_text_field($inputOptions['mailSubject']);
        }

        if (isset($inputOptions['mailText'])) {
            $newOptions['mailText'] = sanitize_text_field($inputOptions['mailText']);
        }

        if (isset($inputOptions['mailTo'])) {
            $newOptions['mailTo'] = sanitize_email($inputOptions['mailTo']);
            if (!$newOptions['mailTo']) {
                $newOptions['mailTo'] = get_option('admin_email');
            }
        }

        $rawOptions = get_post_meta($form->ID, Config::KEY_OPTIONS, true);
        $options    = wp_parse_args($rawOptions, Config::DEFAULT_FORM_OPTIONS);

        $currentOptions = wp_parse_args($newOptions, $options);
        if (count($newOptions['fields'])) {
            $currentOptions['fields'] = $newOptions['fields'];
        }

        update_post_meta($form->ID, Config::KEY_OPTIONS, $currentOptions);

        $result = self::success_response(__("Update successfully", 'forms-rb'));

        $result['options'] = get_post_meta($form->ID, Config::KEY_OPTIONS, true);

        return new \WP_REST_Response($result, 200);
    }

    private static function success_response($data = '')
    {
        return array(
            'code'    => 200,
            'message' => 'SUCCESS',
            'data'    => $data ? $data : __("Operation successfully", 'forms-rb'),
        );
    }

}
new API();
